@@ -1,59 +1,54 @@
 
-/*-
- * Copyright (c) 1995 The Apache Group. All rights reserved.
- * 
+/* ====================================================================
+ * Copyright (c) 1995 The Apache Group.  All rights reserved.
  *
- * Apache httpd license
- * ====================
- * 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * This is the license for the Apache Server. It covers all the
- * files which come in this distribution, and should never be removed.
- * 
- * The "Apache Group" has based this server, called "Apache", on
- * public domain code distributed under the name "NCSA httpd 1.3".
- * 
- * NCSA httpd 1.3 was placed in the public domain by the National Center 
- * for Supercomputing Applications at the University of Illinois 
- * at Urbana-Champaign.
- * 
- * As requested by NCSA we acknowledge,
- * 
- *  "Portions developed at the National Center for Supercomputing
- *   Applications at the University of Illinois at Urbana-Champaign."
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
  *
- * Copyright on the sections of code added by the "Apache Group" belong
- * to the "Apache Group" and/or the original authors. The "Apache Group" and
- * authors hereby grant permission for their code, along with the
- * public domain NCSA code, to be distributed under the "Apache" name.
- * 
- * Reuse of "Apache Group" code outside of the Apache distribution should
- * be acknowledged with the following quoted text, to be included with any new
- * work;
- * 
- * "Portions developed by the "Apache Group", taken with permission 
- *  from the Apache Server   http://www.apache.org/apache/   "
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
  *
+ * 3. All advertising materials mentioning features or use of this
+ *    software must display the following acknowledgment:
+ *    "This product includes software developed by the Apache Group
+ *    for use in the Apache HTTP server project (http://www.apache.org/)."
+ *
+ * 4. The names "Apache Server" and "Apache Group" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission.
+ *
+ * 5. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by the Apache Group
+ *    for use in the Apache HTTP server project (http://www.apache.org/)."
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
+ * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+ * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
+ * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+ * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
+ * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
+ * OF THE POSSIBILITY OF SUCH DAMAGE.
+ * ====================================================================
+ *
+ * This software consists of voluntary contributions made by many
+ * individuals on behalf of the Apache Group and was originally based
+ * on public domain software written at the National Center for
+ * Supercomputing Applications, University of Illinois, Urbana-Champaign.
+ * For more information on the Apache Group and the Apache HTTP server
+ * project, please see <http://www.apache.org/>.
  *
- * Permission is hereby granted to anyone to redistribute Apache under
- * the "Apache" name. We do not grant permission for the resale of Apache, but
- * we do grant permission for vendors to bundle Apache free with other software,
- * or to charge a reasonable price for redistribution, provided it is made
- * clear that Apache is free. Permission is also granted for vendors to 
- * sell support for Apache. We explicitly forbid the redistribution of 
- * Apache under any other name.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
  */
 
 
@@ -68,7 +63,6 @@
 #include "alloc.h"
 
 #include <stdarg.h>
-#include <memory.h>
 
 /*****************************************************************
  *
@@ -380,7 +374,7 @@ void *pcalloc(struct pool *a, int size)
   return res;
 }
 
-char *pstrdup(struct pool *a, char *s)
+char *pstrdup(struct pool *a, const char *s)
 {
   char *res;
   if (s == NULL) return NULL;
@@ -550,6 +544,8 @@ char *table_get (table *t, char *key)
     table_entry *elts = (table_entry *)t->elts;
     int i;
 
+    if (key == NULL) return NULL;
+    
     for (i = 0; i < t->nelts; ++i)
         if (!strcasecmp (elts[i].key, key))
 	    return elts[i].val;
@@ -730,6 +726,19 @@ FILE *pfopen(struct pool *a, char *name, char *mode)
   return fd;
 }
 
+FILE *pfdopen(struct pool *a,int fd,char *mode)
+{
+  FILE *f;
+
+  block_alarms();
+  f=fdopen(fd,mode);
+  if(f != NULL)
+    note_cleanups_for_file(a,f);
+  unblock_alarms();
+  return f;
+}
+
+
 int pfclose(struct pool *a, FILE *fd)
 {
   int res;
@@ -779,7 +788,11 @@ int spawn_child (pool *p, void (*func)(void *), void *data,
 
   block_alarms();
   
-  if (pipe_in && pipe (in_fds) < 0) return 0;
+  if (pipe_in && pipe (in_fds) < 0)
+  {
+      unblock_alarms();
+      return 0;
+  }
   
   if (pipe_out && pipe (out_fds) < 0) {
     if (pipe_in) {
@@ -864,6 +877,15 @@ static void free_proc_chain (struct process_chain *procs)
    * be doing anymore.
    */
 
+#ifndef NEED_WAITPID
+  /* Pick up all defunct processes */
+  for (p = procs; p; p = p->next) {
+    if (waitpid (p->pid, (int *) 0, WNOHANG) > 0) {
+      p->kill_how = kill_never;
+    }
+  }
+#endif
+
   for (p = procs; p; p = p->next) {
     if (p->kill_how == kill_after_timeout) {
       /* Subprocess may be dead already.  Only need the timeout if not. */
