@@ -32,7 +32,7 @@
  * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
- * IT'S CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
  * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
@@ -519,6 +519,11 @@ char *set_pidfile (cmd_parms *cmd, void *dummy, char *arg) {
     return NULL;
 }
 
+char *set_scoreboard (cmd_parms *cmd, void *dummy, char *arg) {
+    scoreboard_fname = pstrdup (cmd->pool, arg);
+    return NULL;
+}
+
 char *set_idcheck (cmd_parms *cmd, void *dummy, int arg) {
     cmd->server->do_rfc931 = arg;
     return NULL;
@@ -531,6 +536,13 @@ char *set_daemons_to_start (cmd_parms *cmd, void *dummy, char *arg) {
 
 char *set_min_free_servers (cmd_parms *cmd, void *dummy, char *arg) {
     daemons_min_free = atoi (arg);
+    if (daemons_min_free <= 0) {
+       fprintf(stderr, "WARNING: detected MinSpareServers set to non-positive.\n");
+       fprintf(stderr, "Resetting to 1 to avoid almost certain Apache failure.\n");
+       fprintf(stderr, "Please read the documentation.\n");
+       daemons_min_free = 1;
+    }
+       
     return NULL;
 }
 
@@ -602,6 +614,8 @@ command_rec core_cmds[] = {
   "the filename of the error log" },
 { "PidFile", set_pidfile, NULL, RSRC_CONF, TAKE1,
     "a file for logging the server process ID"},
+{ "ScoreBoardFile", set_scoreboard, NULL, RSRC_CONF, TAKE1,
+    "a file for apache to maintain runtime process management information"},
 { "AccessConfig", set_server_string_slot,
   (void *)XtOffsetOf (server_rec, access_confname), RSRC_CONF, TAKE1,
   "the filename of the access config file" },
