@@ -1,64 +1,57 @@
 
-/*-
- * Copyright (c) 1995 The Apache Group. All rights reserved.
- * 
+/* ====================================================================
+ * Copyright (c) 1995 The Apache Group.  All rights reserved.
  *
- * Apache httpd license
- * ====================
- * 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * This is the license for the Apache Server. It covers all the
- * files which come in this distribution, and should never be removed.
- * 
- * The "Apache Group" has based this server, called "Apache", on
- * public domain code distributed under the name "NCSA httpd 1.3".
- * 
- * NCSA httpd 1.3 was placed in the public domain by the National Center 
- * for Supercomputing Applications at the University of Illinois 
- * at Urbana-Champaign.
- * 
- * As requested by NCSA we acknowledge,
- * 
- *  "Portions developed at the National Center for Supercomputing
- *   Applications at the University of Illinois at Urbana-Champaign."
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
  *
- * Copyright on the sections of code added by the "Apache Group" belong
- * to the "Apache Group" and/or the original authors. The "Apache Group" and
- * authors hereby grant permission for their code, along with the
- * public domain NCSA code, to be distributed under the "Apache" name.
- * 
- * Reuse of "Apache Group" code outside of the Apache distribution should
- * be acknowledged with the following quoted text, to be included with any new
- * work;
- * 
- * "Portions developed by the "Apache Group", taken with permission 
- *  from the Apache Server   http://www.apache.org/apache/   "
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
  *
+ * 3. All advertising materials mentioning features or use of this
+ *    software must display the following acknowledgment:
+ *    "This product includes software developed by the Apache Group
+ *    for use in the Apache HTTP server project (http://www.apache.org/)."
+ *
+ * 4. The names "Apache Server" and "Apache Group" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission.
+ *
+ * 5. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by the Apache Group
+ *    for use in the Apache HTTP server project (http://www.apache.org/)."
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
+ * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+ * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
+ * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+ * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
+ * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
+ * OF THE POSSIBILITY OF SUCH DAMAGE.
+ * ====================================================================
+ *
+ * This software consists of voluntary contributions made by many
+ * individuals on behalf of the Apache Group and was originally based
+ * on public domain software written at the National Center for
+ * Supercomputing Applications, University of Illinois, Urbana-Champaign.
+ * For more information on the Apache Group and the Apache HTTP server
+ * project, please see <http://www.apache.org/>.
  *
- * Permission is hereby granted to anyone to redistribute Apache under
- * the "Apache" name. We do not grant permission for the resale of Apache, but
- * we do grant permission for vendors to bundle Apache free with other software,
- * or to charge a reasonable price for redistribution, provided it is made
- * clear that Apache is free. Permission is also granted for vendors to 
- * sell support for Apache. We explicitly forbid the redistribution of 
- * Apache under any other name.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * 
  */
 
 
-
 /*
  * httpd.c: simple http daemon for answering WWW file requests
  *
@@ -106,6 +99,7 @@ char *user_name;
 gid_t group_id;
 int max_requests_per_child;
 char *pid_fname;
+char *scoreboard_fname;
 char *server_argv0;
 struct in_addr bind_address;
 int daemons_to_start;
@@ -178,8 +172,8 @@ void accept_mutex_on()
 	continue;
 
     if (ret < 0) {
-	log_error ("Unknown failure grabbing accept lock.  Exiting!",
-		   server_conf);
+	log_unixerr("fcntl", "F_SETLKW", "Error getting accept lock. Exiting!",
+		    server_conf);
 	exit(1);
     }
 }
@@ -188,7 +182,8 @@ void accept_mutex_off()
 {
     if (fcntl (lock_fd, F_SETLKW, &unlock_it) < 0)
     {
-	log_error("Error freeing accept lock.  Exiting!", server_conf);
+	log_unixerr("fcntl", "F_SETLKW", "Error freeing accept lock. Exiting!",
+		    server_conf);
 	exit(1);
     }
 }
@@ -218,7 +213,7 @@ void usage(char *bin)
 
 static conn_rec *current_conn;
 static request_rec *timeout_req;
-static char *timeout_name;
+static char *timeout_name = NULL;
 static int alarms_blocked = 0;
 static int alarm_pending = 0;
 
@@ -242,9 +237,15 @@ int sig;
 #endif
     }
     
-    sprintf(errstr,"%s timed out for %s",
+    if (sig == SIGPIPE) {
+        sprintf(errstr,"%s lost connection to client %s",
 	    timeout_name ? timeout_name : "request",
 	    current_conn->remote_name);
+    } else {
+        sprintf(errstr,"%s timed out for %s",
+	    timeout_name ? timeout_name : "request",
+	    current_conn->remote_name);
+    }
     
     log_error(errstr, current_conn->server);
       
@@ -253,10 +254,20 @@ int sig;
 	 * if it times out...
 	 */
 	
-	log_transaction(timeout_req);
+	request_rec *log_req = timeout_req;
+	
+	while (log_req->main || log_req->prev) {
+	    /* Get back to original request... */
+	    if (log_req->main) log_req = log_req->main;
+	    else log_req = log_req->prev;
+	}
+	
+	log_transaction(log_req);
 
-	fclose (timeout_req->connection->client);
-	fclose (timeout_req->connection->request_in);
+	pfclose (timeout_req->connection->pool,
+		 timeout_req->connection->client);
+	pfclose (timeout_req->connection->pool,
+	         timeout_req->connection->request_in);
     
 	if (!standalone) exit(0);
 #ifdef NEXT
@@ -310,7 +321,21 @@ void kill_timeout (request_rec *dummy) {
     timeout_req = NULL;
     timeout_name = NULL;
 }
- 
+
+/* reset_timeout (request_rec *) resets the timeout in effect,
+ * as long as it hasn't expired already.
+ */
+
+void reset_timeout (request_rec *r) {
+    int i;
+
+    if (timeout_name) { /* timeout has been set */
+        i = alarm(r->server->timeout);
+        if (i == 0) /* timeout already expired, so set it back to 0 */
+	    alarm(0);
+    }
+}
+
 /*****************************************************************
  *
  * Dealing with the scoreboard... a lot of these variables are global
@@ -321,7 +346,6 @@ void kill_timeout (request_rec *dummy) {
  */
 
 static short_score scoreboard_image[HARD_SERVER_MAX];
-static char scoreboard_fname[] = "/tmp/htstatus.XXXXXX";
 static int have_scoreboard_fname = 0;
 static int scoreboard_fd;
 
@@ -357,12 +381,8 @@ static int force_read (int fd, char *buffer, int bufsz)
 
 void reinit_scoreboard (pool *p)
 {
-    if (!have_scoreboard_fname && (mktemp(scoreboard_fname) == NULL ||
-				   scoreboard_fname[0] == '\0')) {
-	fprintf (stderr, "Cannot assign name to scoreboard file!\n");
-	exit (1);
-    }
-    
+    scoreboard_fname = server_root_relative (p, scoreboard_fname);
+
     have_scoreboard_fname = 1;
     
     scoreboard_fd = popenf(p, scoreboard_fname, O_CREAT|O_RDWR, 0644);
@@ -430,7 +450,8 @@ int count_idle_servers ()
     int res = 0;
 
     for (i = 0; i < HARD_SERVER_MAX; ++i)
-	if (scoreboard_image[i].status == SERVER_READY)
+	if (scoreboard_image[i].status == SERVER_READY
+	  || scoreboard_image[i].status == SERVER_STARTING)
 	    ++res;
 
     return res;
@@ -590,7 +611,7 @@ void just_die()			/* SIGHUP to child process??? */
   
 static void set_group_privs()
 {
-  if(!getuid()) {
+  if(!geteuid()) {
     char *name;
   
     /* Get username if passed as a uid */
@@ -610,12 +631,12 @@ static void set_group_privs()
     /* Reset `groups' attributes. */
     
     if (initgroups(name, group_id) == -1) {
-        log_error ("unable to set groups", server_conf);
+	log_unixerr("initgroups", NULL, "unable to set groups", server_conf);
 	exit (1);
     }
 
     if (setgid(group_id) == -1) {
-        log_error ("unable to set group id", server_conf);
+	log_unixerr("setgid", NULL, "unable to set group id", server_conf);
 	exit (1);
     }
   }
@@ -632,8 +653,11 @@ void restart() {
 }
 
 void set_signals() {
-    signal(SIGSEGV,(void (*)())seg_fault);
-    signal(SIGBUS,(void (*)())bus_error);
+	if(!one_process)
+	{
+	    signal(SIGSEGV,(void (*)())seg_fault);
+    	signal(SIGBUS,(void (*)())bus_error);
+	}
     signal(SIGTERM,(void (*)())sig_term);
     signal(SIGHUP,(void (*)())restart);
 }
@@ -646,13 +670,10 @@ void set_signals() {
 pool *pconf;			/* Pool for config stuff */
 pool *ptrans;			/* Pool for per-transaction stuff */
 
-server_rec *find_virtual_server (FILE *client_fd, server_rec *server)
+server_rec *find_virtual_server (struct in_addr server_ip, server_rec *server)
 {
-    struct in_addr server_ip; 
     server_rec *virt;
 
-    server_ip = get_local_addr (fileno(client_fd));
-    
     for (virt = server->next; virt; virt = virt->next)
 	if (virt->host_addr.s_addr == server_ip.s_addr)
 	    return virt;
@@ -688,7 +709,8 @@ void abort_connection (conn_rec *c)
     c->aborted = 1;
 }
 
-conn_rec *new_connection (pool *p, server_rec *server, FILE *in, FILE *out)
+conn_rec *new_connection (pool *p, server_rec *server, FILE *in, FILE *out,
+			  const struct sockaddr_in *saddr)
 {
     conn_rec *conn = (conn_rec *)pcalloc (p, sizeof(conn_rec));
     
@@ -699,7 +721,7 @@ conn_rec *new_connection (pool *p, server_rec *server, FILE *in, FILE *out)
     conn = (conn_rec *)pcalloc(p, sizeof(conn_rec));
     
     conn->pool = p;
-    conn->server = find_virtual_server (in, server);
+    conn->server = find_virtual_server (saddr->sin_addr, server);
     conn->client = out;
     conn->request_in = in;
     
@@ -722,12 +744,8 @@ static int child_num;
 void child_main(int child_num_arg)
 {
     int clen;
-#if defined(NEXT) || defined(LINUX) || defined(SOLARIS2)
-    struct sockaddr_in sa_server;
+    struct sockaddr sa_server;
     struct sockaddr sa_client;
-#else
-    struct sockaddr_in sa_server,sa_client;
-#endif
 
 #ifdef ULTRIX_BRAIN_DEATH
     extern char *rfc931();
@@ -739,20 +757,15 @@ void child_main(int child_num_arg)
     dupped_csd = -1;
     child_num = child_num_arg;
     requests_this_child = 0;
-    update_child_status (child_num, SERVER_READY);
     reopen_scoreboard (pconf);
+    update_child_status (child_num, SERVER_READY);
 
     /* Only try to switch if we're running as root */
-    if(!getuid() && setuid(user_id) == -1) {
-        log_error ("unable to change uid", server_conf);
+    if(!geteuid() && setuid(user_id) == -1) {
+        log_unixerr("setuid", NULL, "unable to change uid", server_conf);
 	exit (1);
     }
 
-    memset((char *) &sa_server, 0, sizeof(sa_server));
-    sa_server.sin_family=AF_INET;
-    sa_server.sin_addr.s_addr=server_conf->host_addr.s_addr;
-    sa_server.sin_port=htons(server_conf->port);
-
 #ifdef NEXT
     setjmp(jmpbuffer);
 #else
@@ -785,44 +798,48 @@ void child_main(int child_num_arg)
 	
 	accept_mutex_on();  /* Lock around "accept", if necessary */
 	
-	while ((csd=accept(sd,(struct sockaddr *)&sa_client,&clen)) == -1) 
+	while ((csd=accept(sd, &sa_client, &clen)) == -1) 
            if (errno != EINTR) 
-		log_error("socket error: accept failed", server_conf);
+		log_unixerr("accept", "(client socket)", NULL, server_conf);
 
 	accept_mutex_off(); /* unlock after "accept" */
+
+	clen = sizeof(sa_server);
+	if(getsockname(csd, &sa_server, &clen) < 0) {
+	    log_unixerr("getsockname", NULL, NULL, server_conf);
+	    continue;
+	}
 	
-	note_cleanups_for_fd (ptrans, csd);
 	dupped_csd = csd;
 #if defined(AUX) || defined(SCO)
 	if ((dupped_csd = dup(csd)) < 0) {
-	    log_error("couldn't duplicate csd", server_conf);
+	    log_unixerr("dup", NULL, "couldn't duplicate csd", server_conf);
 	    dupped_csd = csd;	/* Oh well... */
-	} else
-	    note_cleanups_for_fd (ptrans, dupped_csd);
+	}
 #endif
 	update_child_status (child_num, SERVER_BUSY);
-	conn_in = fdopen (csd, "r");
-	conn_out = fdopen (dupped_csd, "w");
+	conn_in = pfdopen (ptrans, csd, "r");
+	conn_out = pfdopen (ptrans, dupped_csd, "w");
 
-	current_conn = new_connection (ptrans, server_conf, conn_in, conn_out);
+	current_conn = new_connection (ptrans, server_conf, conn_in, conn_out,
+				       (struct sockaddr_in *)&sa_server);
 
 	if (current_conn->server->do_rfc931)
 	    current_conn->remote_logname = 
-		rfc931((struct sockaddr_in *)&sa_client, &sa_server);
+		rfc931((struct sockaddr_in *)&sa_client,
+		       (struct sockaddr_in *)&sa_server);
 	
 	r = read_request (current_conn);
 	if (r) process_request (r); /* else premature EOF --- ignore */
 		
-	if (bytes_in_pool (ptrans) > 80000) {
-	    char errstr[MAX_STRING_LEN];
-	    sprintf (errstr, "Memory hog alert: allocated %ld bytes for %s",
-	             bytes_in_pool (ptrans), r->the_request);
-            log_error (errstr, r->server);
-        }
+	if (bytes_in_pool (ptrans) > 80000)
+	    log_printf(r->server,
+		       "Memory hog alert: allocated %ld bytes for %s",
+		       bytes_in_pool (ptrans), r->the_request);
 		
 	fflush (conn_out);
-	fclose (conn_in);
-	fclose (conn_out);
+	pfclose (ptrans,conn_in);
+	pfclose (ptrans,conn_out);
     }    
 }
 
@@ -837,7 +854,7 @@ void make_child(server_rec *server_conf, int child_num)
     }
 
     if ((pid = fork()) == -1) {
-	log_error("Unable to fork new process", server_conf);
+	log_unixerr("fork", NULL, "Unable to fork new process", server_conf);
 	return;
     } 
     
@@ -873,10 +890,11 @@ void standalone_main(int argc, char **argv)
 
     signal (SIGHUP, SIG_IGN);	/* Until we're done (re)reading config */
     
+    if(!one_process)
 #ifndef NO_KILLPG
-    killpg(pgrp,SIGHUP);	/* Kill 'em off */
+      killpg(pgrp,SIGHUP);	/* Kill 'em off */
 #else
-    kill(-pgrp,SIGHUP);
+      kill(-pgrp,SIGHUP);
 #endif
     
     if (sd != -1) {
@@ -903,13 +921,13 @@ void standalone_main(int argc, char **argv)
 
     note_cleanups_for_fd (pconf, sd); /* arrange to close on exec or restart */
     
-    if((setsockopt(sd,SOL_SOCKET,SO_REUSEADDR,(const char *)&one,sizeof(one)))
+    if((setsockopt(sd,SOL_SOCKET,SO_REUSEADDR,(char *)&one,sizeof(one)))
        == -1) {
         fprintf(stderr,"httpd: could not set socket option\n");
         perror("setsockopt");
         exit(1);
     }
-    if((setsockopt(sd,SOL_SOCKET,SO_KEEPALIVE,(const void *)&keepalive_value,
+    if((setsockopt(sd,SOL_SOCKET,SO_KEEPALIVE,(void *)&keepalive_value,
         sizeof(keepalive_value))) == -1) {
         fprintf(stderr,"httpd: could not set socket option SO_KEEPALIVE\n"); 
         perror("setsockopt"); 
@@ -961,7 +979,10 @@ void standalone_main(int argc, char **argv)
 	if ((count_idle_servers() < daemons_min_free)
 	    && (child_slot = find_free_child_num()) >= 0
 	    && child_slot <= daemons_limit)
+	    {
+	    update_child_status(child_slot,SERVER_STARTING);
 	    make_child(server_conf, child_slot);
+	    }
     }
 
 } /* standalone_main */
@@ -1012,6 +1033,7 @@ main(int argc, char *argv[])
     else {
         conn_rec *conn;
 	request_rec *r;
+	struct sockaddr sa_server;
       
 	open_logs(server_conf, pconf);
 	set_group_privs();
@@ -1020,8 +1042,15 @@ main(int argc, char *argv[])
         user_id = getuid();
         group_id = getgid();
 
-        server_conf->port = get_portnum(fileno(stdout));
-	conn = new_connection (ptrans, server_conf, stdin, stdout);
+	c = sizeof(sa_server);
+	if(getsockname(csd, &sa_server, &c) < 0) {
+	    perror("getsockname");
+	    fprintf(stderr, "Error getting local address\n");
+	    exit(1);
+	}
+	server_conf->port =ntohs(((struct sockaddr_in *)&sa_server)->sin_port);
+	conn = new_connection (ptrans, server_conf, stdin, stdout,
+			       (struct sockaddr_in *)&sa_server);
 	r = read_request (conn);
 	if (r) process_request (r); /* else premature EOF (ignore) */
     }
