@@ -32,7 +32,7 @@
  * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
- * IT'S CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
  * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
@@ -379,6 +379,7 @@ int get_basic_auth_pw (request_rec *r, char **pw)
 
     t = uudecode (r->pool, auth_line);
     r->connection->user = getword (r->pool, &t, ':');
+    r->connection->auth_type = "Basic";
 
     *pw = t;
 
@@ -553,6 +554,8 @@ long send_fd(FILE *f, request_rec *r)
 	
         while(n && !r->connection->aborted) {
             w=fwrite(&buf[o],sizeof(char),n,c->client);
+	    if (w)
+	        reset_timeout(r); /* reset timeout after successfule write */
             n-=w;
             o+=w;
         }
@@ -635,8 +638,8 @@ void send_error_response (request_rec *r, int recursive_error)
 	
         switch (r->status) {
 	case REDIRECT:
-	    fprintf (fd,"The document has moved <A HREF=\"%s\">here</A>.<P>%c",
-		     location, LF);
+	    fprintf (fd,"The document has moved <A HREF=\"%s\">here</A>.<P>\n",
+		     escape_html(r->pool, location));
 	    break;
 	case AUTH_REQUIRED:
 	    fprintf (fd, "This server could not verify that you%c", LF);
@@ -651,28 +654,29 @@ void send_error_response (request_rec *r, int recursive_error)
 	    fprintf (fd, "this server could not understand.<P>%c", LF);
 	    break;
 	case FORBIDDEN:
-	    fprintf (fd, "You don't have permission to access %s%c",
-		     r->uri, LF);
+	    fprintf (fd, "You don't have permission to access %s\n",
+		     escape_html(r->pool, r->uri));
 	    fprintf (fd, "on this server.<P>%c", LF);
 	    break;
 	case NOT_FOUND:
 	    fprintf (fd,
-		     "The requested URL %s was not found on this server.<P>%c",
-		     r->uri, LF);
+		     "The requested URL %s was not found on this server.<P>\n",
+		     escape_html(r->pool, r->uri));
 	    break;
 	case SERVER_ERROR:
 	    fprintf(fd,"The server encountered an internal error or%c",LF);
 	    fprintf(fd,"misconfiguration and was unable to complete%c",LF);
 	    fprintf(fd,"your request.<P>%c",LF);
 	    fprintf(fd,"Please contact the server administrator,%c",LF);
-	    fprintf(fd," %s ", r->server->server_admin);
+	    fprintf(fd," %s ", escape_html(r->pool, r->server->server_admin));
 	    fprintf(fd,"and inform them of the time the error occurred,%c",LF);
 	    fprintf(fd,"and anything you might have done that may have%c",LF);
 	    fprintf(fd,"caused the error.<P>%c",LF);
 	    break;
 	case NOT_IMPLEMENTED:
-	    fprintf(fd,"%s to %s not supported.<P>%c", r->method,
-		    r->uri, LF);
+	    fprintf(fd,"%s to %s not supported.<P>\n",
+		    escape_html(r->pool, r->method),
+		    escape_html(r->pool, r->uri));
 	    break;
 	}
 
