@@ -1,64 +1,57 @@
 
-/*-
- * Copyright (c) 1995 The Apache Group. All rights reserved.
- * 
+/* ====================================================================
+ * Copyright (c) 1995 The Apache Group.  All rights reserved.
  *
- * Apache httpd license
- * ====================
- * 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * This is the license for the Apache Server. It covers all the
- * files which come in this distribution, and should never be removed.
- * 
- * The "Apache Group" has based this server, called "Apache", on
- * public domain code distributed under the name "NCSA httpd 1.3".
- * 
- * NCSA httpd 1.3 was placed in the public domain by the National Center 
- * for Supercomputing Applications at the University of Illinois 
- * at Urbana-Champaign.
- * 
- * As requested by NCSA we acknowledge,
- * 
- *  "Portions developed at the National Center for Supercomputing
- *   Applications at the University of Illinois at Urbana-Champaign."
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
  *
- * Copyright on the sections of code added by the "Apache Group" belong
- * to the "Apache Group" and/or the original authors. The "Apache Group" and
- * authors hereby grant permission for their code, along with the
- * public domain NCSA code, to be distributed under the "Apache" name.
- * 
- * Reuse of "Apache Group" code outside of the Apache distribution should
- * be acknowledged with the following quoted text, to be included with any new
- * work;
- * 
- * "Portions developed by the "Apache Group", taken with permission 
- *  from the Apache Server   http://www.apache.org/apache/   "
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
  *
+ * 3. All advertising materials mentioning features or use of this
+ *    software must display the following acknowledgment:
+ *    "This product includes software developed by the Apache Group
+ *    for use in the Apache HTTP server project (http://www.apache.org/)."
+ *
+ * 4. The names "Apache Server" and "Apache Group" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission.
+ *
+ * 5. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by the Apache Group
+ *    for use in the Apache HTTP server project (http://www.apache.org/)."
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
+ * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+ * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
+ * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+ * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
+ * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
+ * OF THE POSSIBILITY OF SUCH DAMAGE.
+ * ====================================================================
+ *
+ * This software consists of voluntary contributions made by many
+ * individuals on behalf of the Apache Group and was originally based
+ * on public domain software written at the National Center for
+ * Supercomputing Applications, University of Illinois, Urbana-Champaign.
+ * For more information on the Apache Group and the Apache HTTP server
+ * project, please see <http://www.apache.org/>.
  *
- * Permission is hereby granted to anyone to redistribute Apache under
- * the "Apache" name. We do not grant permission for the resale of Apache, but
- * we do grant permission for vendors to bundle Apache free with other software,
- * or to charge a reasonable price for redistribution, provided it is made
- * clear that Apache is free. Permission is also granted for vendors to 
- * sell support for Apache. We explicitly forbid the redistribution of 
- * Apache under any other name.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * 
  */
 
 
-
 /*
  * str.c: string utility things
  * 
@@ -69,6 +62,9 @@
 
 #include "httpd.h"
 #include "http_conf_globals.h"	/* for user_id & group_id */
+#ifdef QNX
+#include <time.h>
+#endif
 
 #ifdef NOTDEF
 extern char** environ;
@@ -142,10 +138,19 @@ struct tm *get_gmtoff(long *tz) {
     t = localtime(&tt);
 #if defined(HAS_GMTOFF)
     *tz = t->tm_gmtoff;
-#else
+#elif !defined(NO_TIMEZONE)
     *tz = - timezone;
     if(t->tm_isdst)
         *tz += 3600;
+#else
+  {
+    static struct tm loc_t;
+
+    loc_t = *t;   /* save it */
+    t = gmtime(&tt);
+    *tz = mktime(&loc_t) - mktime(t);
+    t = &loc_t; /* return pointer to saved time */
+  }
 #endif
     return t;
 }
@@ -192,42 +197,67 @@ int is_matchexp(char *str) {
  */
 void getparents(char *name)
 {
-    int l=0,w=0;
-    const char *lookfor="..";
+    int l, w;
+
+    /* Four paseses, as per RFC 1808 */
+    /* a) remove ./ path segments */
+
+    for (l=0, w=0; name[l] != '\0';)
+    {
+	if (name[l] == '.' && name[l+1] == '/' && (l == 0 || name[l-1] == '/'))
+	    l += 2;
+	else
+	    name[w++] = name[l++];
+    }
+
+    /* b) remove trailing . path, segment */
+    if (w == 1 && name[0] == '.') w--;
+    else if (w > 1 && name[w-1] == '.' && name[w-2] == '/') w--;
+    name[w] = '\0';
+
+    /* c) remove all xx/../ segments. (including leading ../ and /../) */
+    l = 0;
 
     while(name[l]!='\0') {
-        if(name[l]!=lookfor[w]) (w>0 ? (l-=(w-1),w=0) : l++);
-        else {
-            if(lookfor[++w]=='\0') {
-                if((name[l+1]=='\0') ||
-		   ((name[l+1]=='/') &&
-		    (((l > 3) && (name[l-2] == '/'))
-		     || (l<=3)))) {
-                    register int m=l+1,n;
+        if(name[l] == '.' && name[l+1] == '.' && name[l+2] == '/' &&
+	    (l == 0 || name[l-1] == '/')) {
+		register int m=l+3,n;
 
-                    l=l-3;
-                    if(l>=0) {
-                        while((l!=0) && (name[l]!='/')) --l;
-                    }
-                    else l=0;
-                    n=l;
-                    while((name[n]=name[m])) (++n,++m);
-                    w=0;
-                }
-                else w=0;
+		l=l-2;
+		if(l>=0) {
+		    while(l >= 0 && name[l] != '/') l--;
+		    l++;
+		}
+		else l=0;
+		n=l;
+		while((name[n]=name[m])) (++n,++m);
             }
-            else ++l;
-        }
+	else ++l;
     }
-}
+
+    /* d) remove trailing xx/.. segment. */
+    if (l == 2 && name[0] == '.' && name[1] == '.') name[0] = '\0';
+    else if (l > 2 && name[l-1] == '.' && name[l-2] == '.' && name[l-3] == '/')
+    {
+	l = l - 4;
+	if (l >= 0)
+	{
+	    while (l >= 0 && name[l] != '/') l--;
+	    l++;
+	}
+	else l = 0;
+	name[l] = '\0';
+    }
+} 
 
 void no2slash(char *name) {
     register int x,y;
 
-    for(x=0; name[x]; x++)
+    for(x=0; name[x];)
         if(x && (name[x-1] == '/') && (name[x] == '/'))
             for(y=x+1;name[y-1];y++)
                 name[y-1] = name[y];
+	else x++;
 }
 
 char *make_dirstr(pool *p, char *s, int n) {
@@ -406,7 +436,7 @@ char *escape_shell_cmd(pool *p, char *s) {
     strcpy (cmd, s);
     
     for(x=0;cmd[x];x++) {
-        if(ind("&;`'\"|*?~<>^()[]{}$\\",cmd[x]) != -1){
+        if(ind("&;`'\"|*?~<>^()[]{}$\\\n",cmd[x]) != -1){
             for(y=l+1;y>x;y--)
                 cmd[y] = cmd[y-1];
             l++; /* length has been increased */
@@ -439,16 +469,42 @@ char x2c(char *what) {
     return(digit);
 }
 
-void unescape_url(char *url) {
-    register int x,y;
+/*
+ * Unescapes a URL.
+ * Returns 0 on success, non-zero on error
+ * Failure is due to
+ *   bad % escape       returns BAD_REQUEST
+ *
+ *   decoding %00 -> \0
+ *   decoding %2f -> /   (a special character)
+ *                      returns NOT_FOUND
+ */
+int
+unescape_url(char *url) {
+    register int x,y, badesc, badpath;
 
+    badesc = 0;
+    badpath = 0;
     for(x=0,y=0;url[y];++x,++y) {
-        if((url[x] = url[y]) == '%') {
-            url[x] = x2c(&url[y+1]);
-            y+=2;
+	if (url[y] != '%') url[x] = url[y];
+	else
+	{
+	    if (!isxdigit(url[y+1]) || !isxdigit(url[y+2]))
+	    {
+		badesc = 1;
+		url[x] = '%';
+	    } else
+	    {
+		url[x] = x2c(&url[y+1]);
+		y += 2;
+		if (url[x] == '/' || url[x] == '\0') badpath = 1;
+	    }
         }
     }
     url[x] = '\0';
+    if (badesc) return BAD_REQUEST;
+    else if (badpath) return NOT_FOUND;
+    else return OK;
 }
 
 char *construct_url(pool *p, char *uri, server_rec *s) {
@@ -463,7 +519,65 @@ char *construct_url(pool *p, char *uri, server_rec *s) {
     }
 }
 
-#define c2x(what,where) sprintf(where,"%%%2x",what)
+#define c2x(what,where) sprintf(where,"%%%02x",what)
+
+/*
+escape_path_segment() escapes a path segment, as defined in RFC 1808. This
+routine is (should be) OS independent.
+
+os_escape_path() converts an OS path to a URL, in an OS dependent way. In all
+cases if a ':' occurs before the first '/' in the URL, the URL should be
+prefixed with "./" (or the ':' escaped). In the case of Unix, this means
+leaving '/' alone, but otherwise doing what escape_path_segment() does. For
+efficiency reasons, we don't use escape_path_segment(), which is provided for
+reference. Again, RFC 1808 is where this stuff is defined.
+*/
+
+char *escape_path_segment(pool *p, const char *segment) {
+    register int x,y;
+    char *copy = palloc (p, 3 * strlen (segment) + 1);
+            
+    for(x=0,y=0; segment[x]; x++,y++) {
+      char c=segment[x];
+      if((c < 'A' || c > 'Z') && (c < 'a' || c > 'z') && (c < '0' || c >'9')
+	 && ind("$-_.+!*'(),:@&=~",c) == -1)
+	{
+	  c2x(c,&copy[y]);
+	  y+=2;
+	}
+      else
+	copy[y]=c;
+    }
+    copy[y] = '\0';
+    return copy;
+}
+
+char *os_escape_path(pool *p,const char *path) {
+  char *copy=palloc(p,3*strlen(path)+3);
+  char *s=copy;
+  int colon=ind(path,':');
+  int slash=ind(path,'/');
+
+  if(colon >= 0 && (colon < slash || slash < 0))
+    {
+      *s++='.';
+      *s++='/';
+    }
+  for( ; *path ; ++path)
+    {
+      char c=*path;
+      if((c < 'A' || c > 'Z') && (c < 'a' || c > 'z') && (c < '0' || c >'9')
+	 && ind("$-_.+!*'(),:@&=/~",c) == -1)
+	{
+	  c2x(c,s);
+	  s+=3;
+	}
+      else
+	*s++=c;
+    }
+  *s='\0';
+  return copy;
+}
 
 char *escape_uri(pool *p, char *uri) {
     register int x,y;
@@ -479,6 +593,39 @@ char *escape_uri(pool *p, char *uri) {
     return copy;
 }
 
+char *
+escape_html(pool *p, const char *s)
+{
+    int i, j;
+    char *x;
+
+/* first, count the number of extra characters */
+    for (i=0, j=0; s[i] != '\0'; i++)
+	if (s[i] == '<' || s[i] == '>') j += 3;
+	else if (s[i] == '&') j += 4;
+
+    if (j == 0) return pstrdup(p, s);
+    x = palloc(p, i + j + 1);
+    for (i=0, j=0; s[i] != '\0'; i++, j++)
+	if (s[i] == '<')
+	{
+	    memcpy(&x[j], "&lt;", 4);
+	    j += 3;
+	} else if (s[i] == '>')
+	{
+	    memcpy(&x[j], "&gt;", 4);
+	    j += 3;
+	} else if (s[i] == '&')
+	{
+	    memcpy(&x[j], "&amp;", 5);
+	    j += 4;
+	} else
+            x[j] = s[i];
+
+    x[j] = '\0';
+    return x;
+}
+
 #ifdef NOTDEF
 
 void escape_url(char *url) {
@@ -595,10 +742,17 @@ int strncasecmp (const char *a, const char *b, int n)
 #ifdef NEED_INITGROUPS
 int initgroups(const char *name, gid_t basegid)
 {
+#ifdef QNX
+/* QNX does not appear to support supplementary groups.
+Ben <ben@algroup.co.uk> */
+	return 0;
+#else /* ndef QNX */
   gid_t groups[NGROUPS_MAX];
   struct group *g;
   int index = 0;
 
+  setgrent();
+
   groups[index++] = basegid;
 
   while (index < NGROUPS_MAX && ((g = getgrent()) != NULL))
@@ -611,9 +765,12 @@ int initgroups(const char *name, gid_t basegid)
           groups[index++] = g->gr_gid;
     }
 
+  endgrent();
+
   return setgroups(index, groups);
+#endif /* def QNX */
 }
-#endif
+#endif /* def NEED_INITGROUPS */
 
 #ifdef NEED_WAITPID
 /* From ikluft@amdahl.com */
@@ -682,6 +839,7 @@ gid_t gname2id(char *name) {
     else return(ent->gr_gid);
 }
 
+#if 0
 int get_portnum(int sd) {
     struct sockaddr addr;
     int len;
@@ -705,6 +863,7 @@ struct in_addr get_local_addr(int sd) {
          
     return ((struct sockaddr_in *)&addr)->sin_addr;
 }
+#endif
 
 unsigned long get_virthost_addr (char *w, int wild_allowed) {
     struct hostent *hep;
@@ -916,3 +1075,15 @@ char *uudecode(pool *p, char *bufcoded) {
     bufplain[nbytesdecoded] = '\0';
     return bufplain;
 }
+
+#ifdef NEED_STRERROR
+char *
+strerror (int err) {
+
+    char *p;
+    extern char *const sys_errlist[];
+
+    p = sys_errlist[err];
+    return (p);
+}
+#endif
